@@ -11,7 +11,7 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   .chkclass(class(object), must="rma.uni", notav=c("robust.rma", "rma.ls"))
+   .chkclass(class(object), must="rma.uni", notav=c("robust.rma", "rma.ls", "rma.gen"))
 
    if (!missing(parm))
       warning(mstyle$warning("Argument 'parm' (currently) ignored."), call.=FALSE)
@@ -47,7 +47,7 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
    if (!fixed && !random)
       stop(mstyle$stop("At least one of the arguments 'fixed' and 'random' must be TRUE."))
 
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   level <- .level(level)
 
    ddd <- list(...)
 
@@ -70,8 +70,8 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
          type <- "QP"
       }
    } else {
-      #type <- match.arg(type, c("QP", "GENQ", "PL"))
-      if (!is.element(type, c("QP", "GENQ", "PL")))
+      #type <- match.arg(type, c("QP","GENQ","PL"))
+      if (!is.element(type, c("QP","GENQ","PL","Wald","Wald.log","Wald.sqrt")))
          stop(mstyle$stop("Unknown 'type' specified."))
    }
 
@@ -372,8 +372,6 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
       ### PL method ###
       #################
 
-      ### note: cannot actually use this at the moment
-
       if (type == "PL") {
 
          if (con$tau2.min > x$tau2)
@@ -467,6 +465,53 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
 
       ######################################################################
 
+      if (is.element(type, c("Wald","Wald.log","Wald.sqrt"))) {
+         crit    <- qnorm(level/2, lower.tail=FALSE)
+         lb.conv <- TRUE
+         ub.conv <- TRUE
+      }
+
+      ###################
+      ### Wald method ###
+      ###################
+
+      if (type == "Wald") {
+
+         tau2.lb <- x$tau2 - crit * x$se.tau2
+         tau2.ub <- x$tau2 + crit * x$se.tau2
+         tau2.lb <- max(ifelse(is.null(x$control$tau2.min), 0, x$control$tau2.min), tau2.lb)
+
+      }
+
+      #######################
+      ### Wald.log method ###
+      #######################
+
+      if (type == "Wald.log") {
+
+         if (x$tau2 >= 0) {
+            tau2.lb <- exp(log(x$tau2) - crit * x$se.tau2 / x$tau2)
+            tau2.ub <- exp(log(x$tau2) + crit * x$se.tau2 / x$tau2)
+            tau2.ub <- max(x$tau2, tau2.ub) # if tau2 is 0, then CI is 0 to tau2
+         }
+
+      }
+
+      ########################
+      ### Wald.sqrt method ###
+      ########################
+
+      if (type == "Wald.sqrt") {
+
+         if (x$tau2 >= 0) {
+            tau2.lb <- (max(0, sqrt(x$tau2) - crit * x$se.tau2 / (2 * sqrt(x$tau2))))^2
+            tau2.ub <- (sqrt(x$tau2) + crit * x$se.tau2 / (2 * sqrt(x$tau2)))^2
+         }
+
+      }
+
+      ######################################################################
+
       if (!lb.conv)
          warning(mstyle$warning("Error in iterative search for the lower bound."), call.=FALSE)
 
@@ -486,7 +531,7 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
       H2.ub <- tau2.ub / x$vt + 1
 
       tau2 <- c(x$tau2, tau2.lb, tau2.ub)
-      tau  <- sqrt(c(ifelse(x$tau2 >= 0, x$tau2, NA), ifelse(tau2.lb >= 0, tau2.lb, NA), ifelse(tau2.ub >= 0, tau2.ub, NA)))
+      tau  <- sqrt(c(ifelse(x$tau2 >= 0, x$tau2, NA_real_), ifelse(tau2.lb >= 0, tau2.lb, NA_real_), ifelse(tau2.ub >= 0, tau2.ub, NA_real_)))
       I2   <- c(x$I2, I2.lb, I2.ub)
       H2   <- c(x$H2, H2.lb, H2.ub)
 
