@@ -37,11 +37,15 @@ cumul.rma.peto <- function(x, order, digits, transf, targs, progbar=FALSE, ...)
 
    #########################################################################
 
-   if (grepl("^order\\(", deparse(substitute(order))))
+   if (grepl("^order\\(", deparse1(substitute(order))))
       warning(mstyle$warning("Use of order() in 'order' argument is probably erroneous."), call.=FALSE)
 
-   if (missing(order))
+   if (missing(order)) {
       order <- seq_len(x$k.all)
+   } else {
+      mf <- match.call()
+      order <- .getx("order", mf=mf, data=x$data)
+   }
 
    if (length(order) != x$k.all)
       stop(mstyle$stop(paste0("Length of the 'order' argument (", length(order), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
@@ -50,20 +54,24 @@ cumul.rma.peto <- function(x, order, digits, transf, targs, progbar=FALSE, ...)
    ###       original dataset passed to the model fitting function and so we apply
    ###       the same subsetting (if necessary) as was done during model fitting
 
-   if (!is.null(x$subset))
-      order <- order[x$subset]
+   order <- .getsubset(order, x$subset)
 
    order <- order(order, decreasing=decreasing)
 
-   ai.f   <- x$ai.f[order]
-   bi.f   <- x$bi.f[order]
-   ci.f   <- x$ci.f[order]
-   di.f   <- x$di.f[order]
+   ai.f   <- x$outdat.f$ai[order]
+   bi.f   <- x$outdat.f$bi[order]
+   ci.f   <- x$outdat.f$ci[order]
+   di.f   <- x$outdat.f$di[order]
    yi.f   <- x$yi.f[order]
    vi.f   <- x$vi.f[order]
    not.na <- x$not.na[order]
    slab   <- x$slab[order]
    ids    <- x$ids[order]
+   if (inherits(x$data, "environment")) {
+      data <- NULL
+   } else {
+      data <- x$data[order,]
+   }
 
    beta  <- rep(NA_real_, x$k.f)
    se    <- rep(NA_real_, x$k.f)
@@ -76,6 +84,10 @@ cumul.rma.peto <- function(x, order, digits, transf, targs, progbar=FALSE, ...)
    I2    <- rep(NA_real_, x$k.f)
    H2    <- rep(NA_real_, x$k.f)
 
+   ### elements that need to be returned
+
+   outlist <- "beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, QE=QE, QEp=QEp, tau2=tau2, I2=I2, H2=H2"
+
    ### note: skipping NA cases
 
    if (progbar)
@@ -89,7 +101,8 @@ cumul.rma.peto <- function(x, order, digits, transf, targs, progbar=FALSE, ...)
       if (!not.na[i])
          next
 
-      res <- try(suppressWarnings(rma.peto(ai=ai.f, bi=bi.f, ci=ci.f, di=di.f, add=x$add, to=x$to, drop00=x$drop00, level=x$level, subset=seq_len(i))), silent=TRUE)
+      args <- list(ai=ai.f, bi=bi.f, ci=ci.f, di=di.f, add=x$add, to=x$to, drop00=x$drop00, level=x$level, subset=seq_len(i), outlist=outlist)
+      res <- try(suppressWarnings(.do.call(rma.peto, args)), silent=TRUE)
 
       if (inherits(res, "try-error"))
          next
@@ -144,12 +157,14 @@ cumul.rma.peto <- function(x, order, digits, transf, targs, progbar=FALSE, ...)
       out <- list(estimate=beta[not.na], se=se[not.na], zval=zval[not.na], pval=pval[not.na], ci.lb=ci.lb[not.na], ci.ub=ci.ub[not.na], Q=QE[not.na], Qp=QEp[not.na], I2=I2[not.na], H2=H2[not.na])
       out$slab <- slab[not.na]
       out$ids  <- ids[not.na]
+      out$data <- data[not.na,]
    }
 
    if (na.act == "na.exclude" || na.act == "na.pass") {
       out <- list(estimate=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, Q=QE, Qp=QEp, I2=I2, H2=H2)
       out$slab <- slab
       out$ids  <- ids
+      out$data <- data
    }
 
    if (na.act == "na.fail" && any(!x$not.na))
