@@ -71,16 +71,12 @@ fsn <- function(yi, vi, sei, data, type="Rosenthal", alpha=.05, target, weighted
    }
 
    mf <- match.call()
-   mf.yi      <- mf[[match("yi",      names(mf))]]
-   mf.vi      <- mf[[match("vi",      names(mf))]]
-   mf.sei     <- mf[[match("sei",     names(mf))]]
-   #mf.weights <- mf[[match("weights", names(mf))]]
-   mf.subset  <- mf[[match("subset",  names(mf))]]
-   yi      <- eval(mf.yi,      data, enclos=sys.frame(sys.parent()))
-   vi      <- eval(mf.vi,      data, enclos=sys.frame(sys.parent()))
-   sei     <- eval(mf.sei,     data, enclos=sys.frame(sys.parent()))
-   #weights <- eval(mf.weights, data, enclos=sys.frame(sys.parent()))
-   subset  <- eval(mf.subset,  data, enclos=sys.frame(sys.parent()))
+
+   yi     <- .getx("yi",      mf=mf, data=data, checknumeric=TRUE)
+   vi     <- .getx("vi",      mf=mf, data=data, checknumeric=TRUE)
+   sei    <- .getx("sei",     mf=mf, data=data, checknumeric=TRUE)
+   #weight <- .getx("weights", mf=mf, data=data, checknumeric=TRUE)
+   subset <- .getx("subset",  mf=mf, data=data, checknumeric=TRUE)
 
    if (type %in% c("Rosenthal", "Rosenberg", "REM") || (type == "Orwin" && weighted)) {
       if (is.null(vi)) {
@@ -99,12 +95,16 @@ fsn <- function(yi, vi, sei, data, type="Rosenthal", alpha=.05, target, weighted
    if (length(yi) != length(vi))
       stop(mstyle$stop("Length of 'yi' and 'vi' (or 'sei') is not the same."))
 
+   ### check 'vi' argument for potential misuse
+
+   .chkviarg(mf$vi)
+
    ### if a subset of studies is specified
 
    if (!is.null(subset)) {
-      subset <- .setnafalse(subset, k=length(yi))
-      yi <- yi[subset]
-      vi <- vi[subset]
+      subset <- .chksubset(subset, length(yi))
+      yi <- .getsubset(yi, subset)
+      vi <- .getsubset(vi, subset)
    }
 
    ### check for NAs in yi/vi and act accordingly
