@@ -2,12 +2,12 @@ hc.rma.uni <- function(object, digits, transf, targs, control, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   .chkclass(class(object), must="rma.uni", notav=c("rma.ls", "rma.uni.selmodel"))
+   .chkclass(class(object), must="rma.uni", notav=c("rma.ls", "rma.gen", "rma.uni.selmodel"))
 
    x <- object
 
    if (!x$int.only)
-      stop(mstyle$stop("Method only applicable for models without moderators."))
+      stop(mstyle$stop("Method only applicable to models without moderators."))
 
    if (missing(digits)) {
       digits <- .get.digits(xdigits=x$digits, dmiss=TRUE)
@@ -31,7 +31,7 @@ hc.rma.uni <- function(object, digits, transf, targs, control, ...) {
    if (!x$allvipos)
       stop(mstyle$stop("Cannot use method when one or more sampling variances are non-positive."))
 
-   level <- ifelse(x$level == 0, 1, ifelse(x$level >= 1, (100-x$level)/100, ifelse(x$level > .5, 1-x$level, x$level)))
+   level <- .level(x$level)
 
    if (missing(control))
       control <- list()
