@@ -13,10 +13,8 @@
 
       ### for profile and confint, fit model with tau2 fixed to 'val'
 
-      res <- try(suppressWarnings(
-         rma.uni(obj$yi, obj$vi, weights=obj$weights, mods=obj$X, intercept=FALSE,
-                 method=obj$method, weighted=obj$weighted, test=obj$test, level=obj$level,
-                 control=obj$control, tau2=val, skipr2=TRUE, outlist="minimal")), silent=TRUE)
+      args <- list(yi=obj$yi, vi=obj$vi, weights=obj$weights, mods=obj$X, intercept=FALSE, method=obj$method, weighted=obj$weighted, test=obj$test, level=obj$level, control=obj$control, tau2=val, skipr2=TRUE, outlist="minimal")
+      res <- try(suppressWarnings(.do.call(rma.uni, args)), silent=TRUE)
 
    }
 
@@ -82,11 +80,8 @@
 
       } else {
 
-         sav <- try(suppressWarnings(
-            rma.uni(obj$yi, obj$vi, weights=obj$weights, mods=obj$X, intercept=FALSE,
-                    method=obj$method, weighted=obj$weighted, test=obj$test, level=obj$level,
-                    control=obj$control, tau2=ifelse(obj$tau2.fix, obj$tau2, NA),
-                    subset=val, skipr2=TRUE, outlist=outlist)), silent=TRUE)
+         args <- list(yi=obj$yi, vi=obj$vi, weights=obj$weights, mods=obj$X, intercept=FALSE, method=obj$method, weighted=obj$weighted, test=obj$test, level=obj$level, control=obj$control, tau2=ifelse(obj$tau2.fix, obj$tau2, NA), subset=val, skipr2=TRUE, outlist=outlist)
+         sav <- try(suppressWarnings(.do.call(rma.uni, args)), silent=TRUE)
 
       }
 
@@ -131,13 +126,8 @@
       if (comp == "phi")
          phi.arg[phi.pos] <- val
 
-      obj$control$hessian  <- FALSE
-
-      res <- try(suppressWarnings(
-         rma.mv(obj$yi, V=obj$V, W=obj$W, mods=obj$X, random=obj$random, struct=obj$struct, intercept=FALSE,
-                data=obj$mf.r, method=obj$method, test=obj$test, dfs=obj$dfs, level=obj$level, R=obj$R, Rscale=obj$Rscale,
-                sigma2=sigma2.arg, tau2=tau2.arg, rho=rho.arg, gamma2=gamma2.arg, phi=phi.arg, sparse=obj$sparse,
-                dist=obj$dist, control=obj$control, outlist="minimal")), silent=TRUE)
+      args <- list(yi=obj$yi, V=obj$V, W=obj$W, mods=obj$X, random=obj$random, struct=obj$struct, intercept=FALSE, data=obj$mf.r, method=obj$method, test=obj$test, dfs=obj$dfs, level=obj$level, R=obj$R, Rscale=obj$Rscale, sigma2=sigma2.arg, tau2=tau2.arg, rho=rho.arg, gamma2=gamma2.arg, phi=phi.arg, sparse=obj$sparse, dist=obj$dist, control=obj$control, outlist="minimal")
+      res <- try(suppressWarnings(.do.call(rma.mv, args)), silent=TRUE)
 
    }
 
@@ -184,17 +174,14 @@
 
       ### for subset, fit model to subset as specified by 'val'
 
-      if (is.element(obj$measure, c("RR","OR","RD"))) {
-         sav <- try(suppressWarnings(
-            rma.mh(ai=obj$ai, bi=obj$bi, ci=obj$ci, di=obj$di, measure=obj$measure,
-                   add=obj$add, to=obj$to, drop00=obj$drop00, correct=obj$correct,
-                   level=obj$level, subset=val, outlist=outlist)), silent=TRUE)
+      if (is.element(obj$measure, c("RR","OR","RD"))) { # obj$outdat.f$ai[obj$not.na] since obj$outlist$ai values may be modified
+         args <- list(ai=obj$outdat.f$ai[obj$not.na], bi=obj$outdat.f$bi[obj$not.na], ci=obj$outdat.f$ci[obj$not.na], di=obj$outdat.f$di[obj$not.na],
+                      measure=obj$measure, add=obj$add, to=obj$to, drop00=obj$drop00, correct=obj$correct, level=obj$level, subset=val, outlist=outlist)
       } else {
-         sav <- try(suppressWarnings(
-            rma.mh(x1i=obj$x1i, x2i=obj$x2i, t1i=obj$t1i, t2i=obj$t2i, measure=obj$measure,
-                   add=obj$add, to=obj$to, drop00=obj$drop00, correct=obj$correct,
-                   level=obj$level, subset=val, outlist=outlist)), silent=TRUE)
+         args <- list(x1i=obj$outdat.f$x1i[obj$not.na], x2i=obj$outdat.f$x2i[obj$not.na], t1i=obj$outdat.f$t1i[obj$not.na], t2i=obj$outdat.f$t2i[obj$not.na],
+                      measure=obj$measure, add=obj$add, to=obj$to, drop00=obj$drop00, correct=obj$correct, level=obj$level, subset=val, outlist=outlist)
       }
+      sav <- try(suppressWarnings(.do.call(rma.mh, args)), silent=TRUE)
 
    }
 
@@ -211,10 +198,9 @@
 
       ### for subset, fit model to subset as specified by 'val'
 
-      sav <- try(suppressWarnings(
-         rma.peto(ai=obj$ai, bi=obj$bi, ci=obj$ci, di=obj$di,
-                  add=obj$add, to=obj$to, drop00=obj$drop00,
-                  level=obj$level, subset=val, outlist=outlist)), silent=TRUE)
+      args <- list(ai=obj$outdat.f$ai[obj$not.na], bi=obj$outdat.f$bi[obj$not.na], ci=obj$outdat.f$ci[obj$not.na], di=obj$outdat.f$di[obj$not.na],
+                   add=obj$add, to=obj$to, drop00=obj$drop00, level=obj$level, subset=val, outlist=outlist)
+      sav <- try(suppressWarnings(.do.call(rma.peto, args)), silent=TRUE)
 
    }
 
@@ -309,10 +295,8 @@
       if (comp == "alpha")
          alpha.arg[alpha.pos] <- val
 
-      res <- try(suppressWarnings(
-         rma.uni(obj$yi, obj$vi, weights=obj$weights, mods=obj$X, intercept=FALSE, scale=obj$Z, link=obj$link,
-                 method=obj$method, weighted=obj$weighted, test=obj$test, level=obj$level,
-                 control=obj$control, skiphes=TRUE, alpha=alpha.arg, outlist="minimal")), silent=TRUE)
+      args <- list(yi=obj$yi, vi=obj$vi, weights=obj$weights, mods=obj$X, intercept=FALSE, scale=obj$Z, link=obj$link, method=obj$method, weighted=obj$weighted, test=obj$test, level=obj$level, control=obj$control, skiphes=TRUE, alpha=alpha.arg, outlist="minimal")
+      res <- try(suppressWarnings(.do.call(rma.uni, args)), silent=TRUE)
 
    }
 
