@@ -64,10 +64,14 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
             cat(mstyle$section("Mixed-Effects Model"))
          }
          cat(mstyle$section(paste0(" (k = ", x$k, "; ")))
-         if (x$tau2.fix) {
-            cat(mstyle$section("user-specified tau^2 value)"))
+         if (inherits(x, "rma.gen")) {
+            cat(mstyle$section(paste0("estimation method: ", x$method, ")")))
          } else {
-            cat(mstyle$section(paste0("tau^2 estimator: ", x$method, ")")))
+            if (!is.null(x$tau2.fix) && x$tau2.fix) {
+               cat(mstyle$section("user-specified tau^2 value)"))
+            } else {
+               cat(mstyle$section(paste0("tau^2 estimator: ", x$method, ")")))
+            }
          }
       }
 
@@ -84,13 +88,13 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
       names(fs) <- c("logLik", "deviance", "AIC", "BIC", "AICc")
       cat("\n")
       tmp <- capture.output(print(fs, quote=FALSE, print.gap=2))
-      tmp[1] <- paste0(tmp[1], "\u200b")
+      #tmp[1] <- paste0(tmp[1], "\u200b")
       .print.table(tmp, mstyle)
    }
 
    cat("\n")
 
-   if (x$model == "rma.uni" || x$model == "rma.uni.selmodel") {
+   if (x$model == "rma.uni" || x$model == "rma.uni.selmodel" || inherits(x, "rma.gen")) {
 
       if (!is.element(x$method, c("FE","EE","CE"))) {
          if (x$int.only) {
@@ -145,6 +149,17 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
 
    }
 
+   if (inherits(x, "rma.gen")) {
+      cat(mstyle$section("Parameter Estimates:"))
+      cat("\n\n")
+      res.table <- data.frame(as.list(.fcf(x$pars, digits[["var"]])))
+      colnames(res.table) <- names(x$pars)
+      res.table <- res.table[1,,drop=FALSE]
+      tmp <- capture.output(.print.vector(res.table))
+      .print.table(tmp, mstyle)
+      cat("\n")
+   }
+
    if (!is.na(x$QE)) {
       if (x$int.only) {
          cat(mstyle$section("Test for Heterogeneity:"))
@@ -208,7 +223,7 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
    if (is.element(x$test, c("knha","adhoc","t"))) {
       res.table <- data.frame(estimate=.fcf(c(x$beta), digits[["est"]]), se=.fcf(x$se, digits[["se"]]), tval=.fcf(x$zval, digits[["test"]]), df=round(x$ddf,2), pval=.pval(x$pval, digits[["pval"]]), ci.lb=.fcf(x$ci.lb, digits[["ci"]]), ci.ub=.fcf(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
       if (inherits(x, "robust.rma"))
-         colnames(res.table)[c(2:7)] <- paste0(colnames(res.table)[c(2:7)], footsym[1])
+         res.table <- .addfootsym(res.table, 2:7, footsym[1])
    } else {
       res.table <- data.frame(estimate=.fcf(c(x$beta), digits[["est"]]), se=.fcf(x$se, digits[["se"]]), zval=.fcf(x$zval, digits[["test"]]), pval=.pval(x$pval, digits[["pval"]]), ci.lb=.fcf(x$ci.lb, digits[["ci"]]), ci.ub=.fcf(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
    }
@@ -236,7 +251,7 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
    } else {
       tmp <- capture.output(print(res.table, quote=FALSE, right=TRUE, print.gap=2))
    }
-   tmp[1] <- paste0(tmp[1], "\u200b")
+   #tmp[1] <- paste0(tmp[1], "\u200b")
    .print.table(tmp, mstyle)
 
    if (x$model == "rma.ls") {
@@ -285,7 +300,7 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
       } else {
          tmp <- capture.output(print(res.table, quote=FALSE, right=TRUE, print.gap=2))
       }
-      tmp[1] <- paste0(tmp[1], "\u200b")
+      #tmp[1] <- paste0(tmp[1], "\u200b")
       .print.table(tmp, mstyle)
 
    }
@@ -333,7 +348,7 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
       } else {
          tmp <- capture.output(print(res.table, quote=FALSE, right=TRUE, print.gap=2))
       }
-      tmp[1] <- paste0(tmp[1], "\u200b")
+      #tmp[1] <- paste0(tmp[1], "\u200b")
       .print.table(tmp, mstyle)
 
    }
@@ -355,12 +370,12 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
       if (x$robumethod == "default") {
          cat(mstyle$legend(","))
          cat("\n")
-         cat(mstyle$legend(paste0("   approx. ", ifelse(x$int.only, "t-test and confidence interval", "t/F-tests and confidence intervals"), ", dfs = residual method)")))
+         cat(mstyle$legend(paste0("   approx ", ifelse(x$int.only, "t-test and confidence interval", "t/F-tests and confidence intervals"), ", df: residual method)")))
       } else {
          if (x$coef_test == "Satterthwaite" && x$conf_test == "Satterthwaite" && x$wald_test == "HTZ") {
             cat(mstyle$legend(","))
             cat("\n")
-            cat(mstyle$legend(paste0("   approx. ", ifelse(x$int.only, "t-test and confidence interval", "t/F-tests and confidence intervals"), ", dfs = Satterthwaite method)")))
+            cat(mstyle$legend(paste0("   approx ", ifelse(x$int.only, "t-test and confidence interval", "t/F-tests and confidence intervals"), ", df: Satterthwaite approx)")))
          } else {
             cat(mstyle$legend(")"))
          }
