@@ -1,7 +1,7 @@
 regplot.rma <- function(x, mod, pred=TRUE, ci=TRUE, pi=FALSE, shade=TRUE,
 xlim, ylim, predlim, olim, xlab, ylab, at, digits=2L,
 transf, atransf, targs, level=x$level,
-pch=21, psize, plim=c(0.5,3), col="black", bg="darkgray",
+pch, psize, plim=c(0.5,3), col, bg, slab,
 grid=FALSE, refline, label=FALSE, offset=c(1,1), labsize=1,
 lcol, lwd, lty, legend=FALSE, xvals, ...) {
 
@@ -15,7 +15,7 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
       stop(mstyle$stop("Plot not applicable to intercept-only models."))
 
    na.act <- getOption("na.action")
-   on.exit(options(na.action=na.act))
+   on.exit(options(na.action=na.act), add=TRUE)
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
@@ -26,12 +26,53 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
    if (missing(atransf))
       atransf <- FALSE
 
-   transf.char  <- deparse(substitute(transf))
-   atransf.char <- deparse(substitute(atransf))
+   transf.char  <- deparse(transf)
+   atransf.char <- deparse(atransf)
 
    if (is.function(transf) && is.function(atransf))
       stop(mstyle$stop("Use either 'transf' or 'atransf' to specify a transformation (not both)."))
 
+   mf <- match.call()
+
+   if (missing(pch)) {
+      pch <- 21
+   } else {
+      pch <- .getx("pch", mf=mf, data=x$data)
+   }
+
+   if (missing(psize)) {
+      psize <- NULL
+   } else {
+      psize <- .getx("psize", mf=mf, data=x$data)
+   }
+
+   if (missing(col)) {
+      col <- "black"
+   } else {
+      col <- .getx("col", mf=mf, data=x$data)
+   }
+
+   if (missing(bg)) {
+      bg <- "darkgray"
+   } else {
+      bg <- .getx("bg", mf=mf, data=x$data)
+   }
+
+   if (missing(slab)) {
+      slab <- x$slab
+   } else {
+      slab <- .getx("slab", mf=mf, data=x$data)
+      if (length(slab) != x$k.all)
+         stop(mstyle$stop(paste0("Length of the 'slab' argument (", length(slab), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
+      slab <- .getsubset(slab, x$subset)
+   }
+
+   if (missing(label)) {
+      label <- NULL
+   } else {
+      label <- .getx("label", mf=mf, data=x$data)
+   }
+
    if (missing(targs))
       targs <- NULL
 
@@ -41,12 +82,6 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
    if (missing(at))
       at <- NULL
 
-   if (missing(psize))
-      psize <- NULL
-
-   if (missing(label))
-      label <- NULL
-
    ### grid argument can either be a logical or a color
 
    if (is.logical(grid))
@@ -72,7 +107,7 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
    if (inherits(pred, "list.rma")) {
       addpred <- TRUE
       if (missing(xvals))
-         stop(mstyle$stop("Need to specify the 'xvals' argument."))
+         stop(mstyle$stop("Must specify 'xvals' argument when passing an object from predict() to 'pred'."))
       if (length(xvals) != length(pred$pred))
          stop(mstyle$stop(paste0("Length of the 'xvals' argument (", length(xvals), ") does not correspond to the number of predicted values (", length(pred$pred), ").")))
    } else {
@@ -90,7 +125,7 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
       lcol <- c("black", "black", "black", "gray40")
    } else {
       if (length(lcol) == 1L)
-         lcol <- rep(lcol, 4)
+         lcol <- rep(lcol, 4L)
       if (length(lcol) == 2L)
          lcol <- c(lcol[c(1,2,2)], "gray40")
       if (length(lcol) == 3L)
@@ -101,7 +136,7 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
       lty <- c("solid", "dashed", "dotted", "solid")
    } else {
       if (length(lty) == 1L)
-         lty <- rep(lty, 4)
+         lty <- rep(lty, 4L)
       if (length(lty) == 2L)
          lty <- c(lty[c(1,2,2)], "solid")
       if (length(lty) == 3L)
@@ -112,14 +147,37 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
       lwd <- c(3,1,1,2)
    } else {
       if (length(lwd) == 1L)
-         lwd <- rep(lwd, 4)
+         lwd <- rep(lwd, 4L)
       if (length(lwd) == 2L)
          lwd <- c(lwd[c(1,2,2)], 2)
       if (length(lwd) == 3L)
          lwd <- c(lwd, 2)
    }
 
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   level <- .level(level)
+
+   ddd <- list(...)
+
+   lplot    <- function(..., grep, fixed) plot(...)
+   laxis    <- function(..., grep, fixed) axis(...)
+   lpolygon <- function(..., grep, fixed) polygon(...)
+   llines   <- function(..., grep, fixed) lines(...)
+   lpoints  <- function(..., grep, fixed) points(...)
+   labline  <- function(..., grep, fixed) abline(...)
+   lbox     <- function(..., grep, fixed) box(...)
+   ltext    <- function(..., grep, fixed) text(...)
+
+   if (is.null(ddd$fixed)) {
+      fixed <- FALSE
+   } else {
+      fixed <- .isTRUE(ddd$fixed)
+   }
+
+   if (is.null(ddd$grep)) {
+      grep <- FALSE
+   } else {
+      grep <- .isTRUE(ddd$grep)
+   }
 
    ############################################################################
 
@@ -129,7 +187,11 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
       if (x$p == 2L && x$int.incl) {
          mod <- 2
       } else {
-         stop(mstyle$stop("Need to specify the 'mod' argument."))
+         if (x$p == 1L) {
+            mod <- 1
+         } else {
+            stop(mstyle$stop("Must specify 'mod' argument for models with multiple predictors."))
+         }
       }
    }
 
@@ -141,13 +203,21 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
 
    if (is.character(mod)) {
 
-      mod.pos <- charmatch(mod, colnames(x$X))
+      if (grep) {
 
-      if (is.na(mod.pos))
-         stop(mstyle$stop("Argument 'mod' must be the name of a moderator variable in the model."))
+         mod.pos <- grep(mod, colnames(x$X), fixed=fixed)
 
-      if (mod.pos == 0L)
-         stop(mstyle$stop("No ambiguous match found for variable name specified via 'mod' argument."))
+         if (length(mod.pos) != 1L)
+            stop(mstyle$stop("Could not find or uniquely identify the moderator variable specified via the 'mod' argument."))
+
+      } else {
+
+         mod.pos <- charmatch(mod, colnames(x$X))
+
+         if (is.na(mod.pos) || mod.pos == 0L)
+            stop(mstyle$stop("Could not find or uniquely identify the moderator variable specified via the 'mod' argument."))
+
+      }
 
    } else {
 
@@ -158,12 +228,11 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
 
    }
 
-   ### extract the observed outcomes, corresponding sampling variances, model matrix, slab, and ids
+   ### extract the observed outcomes, corresponding sampling variances, model matrix, and ids
 
    yi   <- c(x$yi.f)
    vi   <- x$vi.f
    X    <- x$X.f
-   slab <- x$slab
    ids  <- x$ids
 
    ### get weights
@@ -188,8 +257,7 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
    if (length(pch) != x$k.all)
       stop(mstyle$stop(paste0("Length of the 'pch' argument (", length(pch), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-   if (!is.null(x$subset))
-      pch <- pch[x$subset]
+   pch <- .getsubset(pch, x$subset)
 
    psize.char <- FALSE
 
@@ -215,8 +283,7 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
          if (length(psize) != x$k.all)
             stop(mstyle$stop(paste0("Length of the 'psize' argument (", length(psize), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-         if (!is.null(x$subset))
-            psize <- psize[x$subset]
+         psize <- .getsubset(psize, x$subset)
 
       }
 
@@ -228,8 +295,7 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
    if (length(col) != x$k.all)
       stop(mstyle$stop(paste0("Length of the 'col' argument (", length(col), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-   if (!is.null(x$subset))
-      col <- col[x$subset]
+   col <- .getsubset(col, x$subset)
 
    if (length(bg) == 1L)
       bg <- rep(bg, x$k.all)
@@ -237,8 +303,7 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
    if (length(bg) != x$k.all)
       stop(mstyle$stop(paste0("Length of the 'bg' argument (", length(bg), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-   if (!is.null(x$subset))
-      bg <- bg[x$subset]
+   bg <- .getsubset(bg, x$subset)
 
    if (!is.null(label)) {
 
@@ -250,15 +315,14 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
 
             label <- rep(TRUE, x$k.all)
 
-            if (!is.null(x$subset))
-               label <- label[x$subset]
+            label <- .getsubset(label, x$subset)
 
          }
 
       } else if (is.logical(label)) {
 
-         if (!is.logical(label))
-            stop(mstyle$stop("Argument 'label' must be a logical vector (or a single character string)."))
+         #if (!is.logical(label))
+         #   stop(mstyle$stop("Argument 'label' must be a logical vector (or a single character string)."))
 
          if (length(label) == 1L)
             label <- rep(label, x$k.all)
@@ -266,14 +330,15 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
          if (length(label) != x$k.all)
             stop(mstyle$stop(paste0("Length of the 'label' argument (", length(label), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-         if (!is.null(x$subset))
-            label <- label[x$subset]
+         label <- .getsubset(label, x$subset)
 
       } else if (is.numeric(label)) {
 
          label <- round(label)
          label <- seq(x$k.all) %in% label
 
+         label <- .getsubset(label, x$subset)
+
       }
 
    }
@@ -536,7 +601,7 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
 
    ### set up plot
 
-   plot(NA, xlab=xlab, ylab=ylab, xlim=xlim, ylim=ylim, yaxt="n", ...)
+   lplot(NA, xlab=xlab, ylab=ylab, xlim=xlim, ylim=ylim, yaxt="n", ...)
 
    ### generate y-axis positions if none are specified
 
@@ -563,25 +628,25 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
 
    ### add y-axis
 
-   axis(side=2, at=at, labels=at.lab, ...)
+   laxis(side=2, at=at, labels=at.lab, ...)
 
    ### add predicted values / CI bounds
 
    if (shade) {
       if (pi)
-         polygon(c(xs, rev(xs)), c(pi.lb, rev(pi.ub)), border=NA, col=shadecol[2], ...)
+         lpolygon(c(xs, rev(xs)), c(pi.lb, rev(pi.ub)), border=NA, col=shadecol[2], ...)
       if (ci)
-         polygon(c(xs, rev(xs)), c(ci.lb, rev(ci.ub)), border=NA, col=shadecol[1], ...)
+         lpolygon(c(xs, rev(xs)), c(ci.lb, rev(ci.ub)), border=NA, col=shadecol[1], ...)
    }
 
    if (ci) {
-      lines(xs, ci.lb, col=lcol[2], lty=lty[2], lwd=lwd[2], ...)
-      lines(xs, ci.ub, col=lcol[2], lty=lty[2], lwd=lwd[2], ...)
+      llines(xs, ci.lb, col=lcol[2], lty=lty[2], lwd=lwd[2], ...)
+      llines(xs, ci.ub, col=lcol[2], lty=lty[2], lwd=lwd[2], ...)
    }
 
    if (pi) {
-      lines(xs, pi.lb, col=lcol[3], lty=lty[3], lwd=lwd[3], ...)
-      lines(xs, pi.ub, col=lcol[3], lty=lty[3], lwd=lwd[3], ...)
+      llines(xs, pi.lb, col=lcol[3], lty=lty[3], lwd=lwd[3], ...)
+      llines(xs, pi.ub, col=lcol[3], lty=lty[3], lwd=lwd[3], ...)
    }
 
    ### add grid
@@ -591,14 +656,14 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
 
    ### add refline
 
-   abline(h=refline, col=lcol[4], lty=lty[4], lwd=lwd[4], ...)
+   labline(h=refline, col=lcol[4], lty=lty[4], lwd=lwd[4], ...)
 
    if (addpred)
-      lines(xs, pred, col=lcol[1], lty=lty[1], lwd=lwd[1], ...)
+      llines(xs, pred, col=lcol[1], lty=lty[1], lwd=lwd[1], ...)
 
    ### redraw box
 
-   box(...)
+   lbox(...)
 
    ### order points by psize for plotting
 
@@ -613,7 +678,7 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
 
    ### add points
 
-   points(x=xi.o, y=yi.o, pch=pch.o, col=col.o, bg=bg.o, cex=psize.o, ...)
+   lpoints(x=xi.o, y=yi.o, pch=pch.o, col=col.o, bg=bg.o, cex=psize.o, ...)
 
    ### labeling of points
 
@@ -641,9 +706,9 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
       for (i in which(label)) {
 
          if (isTRUE(yi[i] > yi.pred[i])) { # yi.pred might be NULL, so use isTRUE()
-            text(xi[i], yi[i] + offset[1] + offset[2]*psize[i]^offset[3], slab[i], cex=labsize, ...)
+            ltext(xi[i], yi[i] + offset[1] + offset[2]*psize[i]^offset[3], slab[i], cex=labsize, ...)
          } else {
-            text(xi[i], yi[i] - offset[1] - offset[2]*psize[i]^offset[3], slab[i], cex=labsize, ...)
+            ltext(xi[i], yi[i] - offset[1] - offset[2]*psize[i]^offset[3], slab[i], cex=labsize, ...)
          }
 
       }
@@ -773,6 +838,10 @@ lcol, lwd, lty, legend=FALSE, xvals, ...) {
    ############################################################################
 
    sav <- data.frame(slab, ids, xi, yi, pch, psize, col, bg, label, order=order.vec)
+   if (length(yi.pred) != 0L) # yi.pred might be NULL or list()
+      sav$pred <- yi.pred
+   attr(sav, "offset")  <- offset
+   attr(sav, "labsize") <- labsize
    class(sav) <- "regplot"
    invisible(sav)
 
