@@ -1,14 +1,29 @@
-replmiss <- function(x, y) {
+replmiss <- function(x, y, data) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (length(y) == 0L)
-      y <- NA
+   ### check if data argument has been specified
 
-   ### catch cases where x is of length 0
+   if (missing(data))
+      data <- NULL
+
+   if (is.null(data)) {
+      data <- sys.frame(sys.parent())
+   } else {
+      if (!is.data.frame(data))
+         data <- data.frame(data)
+   }
+
+   mf <- match.call()
+
+   x <- .getx("x", mf=mf, data=data, checknull=FALSE)
+   y <- .getx("y", mf=mf, data=data, checknull=FALSE)
+
+   if (length(y) == 0L)
+      return(x)
 
    if (length(x) == 0L)
-      return(x)
+      x <- rep(NA, length(y))
 
    ### in case user specifies a constant to use for replacement
 
@@ -21,7 +36,8 @@ replmiss <- function(x, y) {
       stop(mstyle$stop("Length of 'x' and 'y' is not the same."))
 
    #x <- ifelse(is.na(x), y, x) # this is quite a bit slower than the following
-   x[is.na(x)] <- y[is.na(x)]
+   is.na.x <- is.na(x)
+   x[is.na.x] <- y[is.na.x]
 
    return(x)
 
