@@ -1,7 +1,7 @@
 rma.mh   <- function(ai, bi, ci, di, n1i, n2i, x1i, x2i, t1i, t2i, measure="OR",
 data, slab, subset,
 add=1/2, to="only0", drop00=TRUE, ### for add/to/drop00, 1st element for escalc(), 2nd for MH method
-correct=TRUE, level=95, digits, verbose=FALSE, ...) {
+correct=TRUE, level=95, verbose=FALSE, digits, ...) {
 
    #########################################################################
 
@@ -69,13 +69,15 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
 
    if (verbose > 2) {
       opwarn <- options(warn=1)
-      on.exit(options(warn=opwarn$warn))
+      on.exit(options(warn=opwarn$warn), add=TRUE)
    }
 
    #########################################################################
 
+   if (verbose) .space()
+
    if (verbose)
-      message(mstyle$message("\nExtracting data and computing yi/vi values ..."))
+      message(mstyle$message("Extracting data and computing yi/vi values ..."))
 
    ### check if data argument has been specified
 
@@ -93,10 +95,8 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
 
    ### extract slab and subset values, possibly from the data frame specified via data (arguments not specified are NULL)
 
-   mf.slab   <- mf[[match("slab",   names(mf))]]
-   mf.subset <- mf[[match("subset", names(mf))]]
-   slab   <- eval(mf.slab,   data, enclos=sys.frame(sys.parent()))
-   subset <- eval(mf.subset, data, enclos=sys.frame(sys.parent()))
+   slab   <- .getx("slab",   mf=mf, data=data)
+   subset <- .getx("subset", mf=mf, data=data)
 
    #########################################################################
 
@@ -104,20 +104,15 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
 
    if (is.element(measure, c("RR","OR","RD"))) {
 
-      x1i <- x2i <- t1i <- t2i <- x1i.f <- x2i.f <- t1i.f <- t2i.f <- NA
+      x1i <- x2i <- t1i <- t2i <- NA
+
+      ai  <- .getx("ai",  mf=mf, data=data, checknumeric=TRUE)
+      bi  <- .getx("bi",  mf=mf, data=data, checknumeric=TRUE)
+      ci  <- .getx("ci",  mf=mf, data=data, checknumeric=TRUE)
+      di  <- .getx("di",  mf=mf, data=data, checknumeric=TRUE)
+      n1i <- .getx("n1i", mf=mf, data=data, checknumeric=TRUE)
+      n2i <- .getx("n2i", mf=mf, data=data, checknumeric=TRUE)
 
-      mf.ai  <- mf[[match("ai",  names(mf))]]
-      mf.bi  <- mf[[match("bi",  names(mf))]]
-      mf.ci  <- mf[[match("ci",  names(mf))]]
-      mf.di  <- mf[[match("di",  names(mf))]]
-      mf.n1i <- mf[[match("n1i", names(mf))]]
-      mf.n2i <- mf[[match("n2i", names(mf))]]
-      ai  <- eval(mf.ai,  data, enclos=sys.frame(sys.parent()))
-      bi  <- eval(mf.bi,  data, enclos=sys.frame(sys.parent()))
-      ci  <- eval(mf.ci,  data, enclos=sys.frame(sys.parent()))
-      di  <- eval(mf.di,  data, enclos=sys.frame(sys.parent()))
-      n1i <- eval(mf.n1i, data, enclos=sys.frame(sys.parent()))
-      n2i <- eval(mf.n2i, data, enclos=sys.frame(sys.parent()))
       if (is.null(bi)) bi <- n1i - ai
       if (is.null(di)) di <- n2i - ci
       ni <- ai + bi + ci + di
@@ -125,6 +120,9 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
       k <- length(ai) ### number of outcomes before subsetting
       k.all <- k
 
+      if (length(ai)==0L || length(bi)==0L || length(ci)==0L || length(di)==0L)
+         stop(mstyle$stop("Must specify arguments ai, bi, ci, di (or ai, ci, n1i, n2i) for this measure."))
+
       ids <- seq_len(k)
 
       ### generate study labels if none are specified
@@ -159,16 +157,17 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
          if (verbose)
             message(mstyle$message("Subsetting ..."))
 
-         subset <- .setnafalse(subset, k=k)
+         subset <- .chksubset(subset, k)
 
-         ai   <- ai[subset]
-         bi   <- bi[subset]
-         ci   <- ci[subset]
-         di   <- di[subset]
-         ni   <- ni[subset]
-         slab <- slab[subset]
-         ids  <- ids[subset]
-         k    <- length(ai)
+         ai   <- .getsubset(ai,   subset)
+         bi   <- .getsubset(bi,   subset)
+         ci   <- .getsubset(ci,   subset)
+         di   <- .getsubset(di,   subset)
+         ni   <- .getsubset(ni,   subset)
+         slab <- .getsubset(slab, subset)
+         ids  <- .getsubset(ids,  subset)
+
+         k <- length(ai)
 
       }
 
@@ -179,9 +178,9 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
 
       ### calculate observed effect estimates and sampling variances
 
-      dat <- escalc(measure=measure, ai=ai, bi=bi, ci=ci, di=di, add=add[1], to=to[1], drop00=drop00[1], onlyo1=onlyo1, addyi=addyi, addvi=addvi)
-      yi  <- dat$yi ### one or more yi/vi pairs may be NA/NA
-      vi  <- dat$vi ### one or more yi/vi pairs may be NA/NA
+      dat <- .do.call(escalc, measure=measure, ai=ai, bi=bi, ci=ci, di=di, add=add[1], to=to[1], drop00=drop00[1], onlyo1=onlyo1, addyi=addyi, addvi=addvi)
+      yi <- dat$yi ### one or more yi/vi pairs may be NA/NA
+      vi <- dat$vi ### one or more yi/vi pairs may be NA/NA
 
       ### if drop00[2]=TRUE, set counts to NA for studies that have no events (or all events) in both arms
 
@@ -196,10 +195,8 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
 
       ### save the actual cell frequencies and yi/vi values (including potential NAs)
 
-      ai.f <- ai
-      bi.f <- bi
-      ci.f <- ci
-      di.f <- di
+      outdat.f <- list(ai=ai, bi=bi, ci=ci, di=di)
+
       yi.f <- yi
       vi.f <- vi
       ni.f <- ni
@@ -320,21 +317,21 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
 
    if (is.element(measure, c("IRR","IRD"))) {
 
-      ai <- bi <- ci <- di <- ai.f <- bi.f <- ci.f <- di.f <- NA
+      ai <- bi <- ci <- di <- NA
+
+      x1i <- .getx("x1i", mf=mf, data=data)
+      x2i <- .getx("x2i", mf=mf, data=data)
+      t1i <- .getx("t1i", mf=mf, data=data)
+      t2i <- .getx("t2i", mf=mf, data=data)
 
-      mf.x1i <- mf[[match("x1i", names(mf))]]
-      mf.x2i <- mf[[match("x2i", names(mf))]]
-      mf.t1i <- mf[[match("t1i", names(mf))]]
-      mf.t2i <- mf[[match("t2i", names(mf))]]
-      x1i <- eval(mf.x1i, data, enclos=sys.frame(sys.parent()))
-      x2i <- eval(mf.x2i, data, enclos=sys.frame(sys.parent()))
-      t1i <- eval(mf.t1i, data, enclos=sys.frame(sys.parent()))
-      t2i <- eval(mf.t2i, data, enclos=sys.frame(sys.parent()))
       ni  <- t1i + t2i
 
       k <- length(x1i) ### number of outcomes before subsetting
       k.all <- k
 
+      if (length(x1i)==0L || length(x2i)==0L || length(t1i)==0L || length(t2i)==0L)
+         stop(mstyle$stop("Must specify arguments x1i, x2i, t1i, t2i for this measure."))
+
       ids <- seq_len(k)
 
       ### generate study labels if none are specified
@@ -366,16 +363,17 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
          if (verbose)
             message(mstyle$message("Subsetting ..."))
 
-         subset <- .setnafalse(subset, k=k)
+         subset <- .chksubset(subset, k)
 
-         x1i  <- x1i[subset]
-         x2i  <- x2i[subset]
-         t1i  <- t1i[subset]
-         t2i  <- t2i[subset]
-         ni   <- ni[subset]
-         slab <- slab[subset]
-         ids  <- ids[subset]
-         k    <- length(x1i)
+         x1i  <- .getsubset(x1i,  subset)
+         x2i  <- .getsubset(x2i,  subset)
+         t1i  <- .getsubset(t1i,  subset)
+         t2i  <- .getsubset(t2i,  subset)
+         ni   <- .getsubset(ni,   subset)
+         slab <- .getsubset(slab, subset)
+         ids  <- .getsubset(ids,  subset)
+
+         k <- length(x1i)
 
       }
 
@@ -386,7 +384,7 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
 
       ### calculate observed effect estimates and sampling variances
 
-      dat <- escalc(measure=measure, x1i=x1i, x2i=x2i, t1i=t1i, t2i=t2i, add=add[1], to=to[1], drop00=drop00[1], onlyo1=onlyo1, addyi=addyi, addvi=addvi)
+      dat <- .do.call(escalc, measure=measure, x1i=x1i, x2i=x2i, t1i=t1i, t2i=t2i, add=add[1], to=to[1], drop00=drop00[1], onlyo1=onlyo1, addyi=addyi, addvi=addvi)
       yi  <- dat$yi ### one or more yi/vi pairs may be NA/NA
       vi  <- dat$vi ### one or more yi/vi pairs may be NA/NA
 
@@ -401,13 +399,11 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
 
       ### save the actual cell frequencies and yi/vi values (including potential NAs)
 
-      x1i.f <- x1i
-      x2i.f <- x2i
-      t1i.f <- t1i
-      t2i.f <- t2i
-      yi.f  <- yi
-      vi.f  <- vi
-      ni.f  <- ni
+      outdat.f <- list(x1i=x1i, x2i=x2i, t1i=t1i, t2i=t2i)
+
+      yi.f <- yi
+      vi.f <- vi
+      ni.f <- ni
 
       k.f <- k ### total number of tables including all NAs
 
@@ -513,7 +509,7 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
 
    #########################################################################
 
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   level <- .level(level)
 
    CO <- COp <- MH <- MHp <- BD <- BDp <- TA <- TAp <- k.pos <- NA
 
@@ -732,7 +728,11 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
 
       ll.ML     <- -1/2 * (k.yi)   * log(2*base::pi)                   - 1/2 * sum(log(vi))                      - 1/2 * QE
       ll.REML   <- -1/2 * (k.yi-1) * log(2*base::pi) + 1/2 * log(k.yi) - 1/2 * sum(log(vi)) - 1/2 * log(sum(wi)) - 1/2 * QE
-      dev.ML    <- -2 * (ll.ML - sum(dnorm(yi, mean=yi, sd=sqrt(vi), log=TRUE)))
+      if (any(vi <= 0)) {
+         dev.ML <- -2 * ll.ML
+      } else {
+         dev.ML <- -2 * (ll.ML - sum(dnorm(yi, mean=yi, sd=sqrt(vi), log=TRUE)))
+      }
       AIC.ML    <- -2 * ll.ML   + 2
       BIC.ML    <- -2 * ll.ML   + log(k.yi)
       AICc.ML   <- -2 * ll.ML   + 2 * max(k.yi, 3) / (max(k.yi, 3) - 2)
@@ -774,7 +774,9 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
    test      <- "z"
    ddf       <- NA
 
-   if (is.null(ddd$outlist)) {
+   if (is.null(ddd$outlist) || ddd$outlist == "nodata") {
+
+      outdat <- list(ai=ai, bi=bi, ci=ci, di=di, x1i=x1i, x2i=x2i, t1i=t1i, t2i=t2i)
 
       res <- list(b=beta, beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, vb=vb,
                   tau2=tau2, tau2.f=tau2,
@@ -783,8 +785,7 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
                   k=k, k.f=k.f, k.yi=k.yi, k.pos=k.pos, k.eff=k.eff, k.all=k.all, p=p, p.eff=p.eff, parms=parms,
                   int.only=int.only, intercept=intercept, coef.na=coef.na,
                   yi=yi, vi=vi, yi.f=yi.f, vi.f=vi.f, X.f=X.f,
-                  ai=ai, bi=bi, ci=ci, di=di, ai.f=ai.f, bi.f=bi.f, ci.f=ci.f, di.f=di.f,
-                  x1i=x1i, x2i=x2i, t1i=t1i, t2i=t2i, x1i.f=x1i.f, x2i.f=x2i.f, t1i.f=t1i.f, t2i.f=t2i.f, ni=ni, ni.f=ni.f,
+                  outdat.f=outdat.f, outdat=outdat, ni=ni, ni.f=ni.f,
                   ids=ids, not.na=not.na, subset=subset, not.na.yivi=not.na.yivi, slab=slab, slab.null=slab.null,
                   measure=measure, method=method, weighted=weighted,
                   test=test, ddf=ddf, dfs=ddf, btt=btt, m=m,
@@ -793,9 +794,11 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
                   fit.stats=fit.stats,
                   formula.yi=NULL, formula.mods=NULL, version=packageVersion("metafor"), call=mf)
 
-   }
+      if (is.null(ddd$outlist))
+         res <- append(res, list(data=data), which(names(res) == "fit.stats"))
+
+   } else {
 
-   if (!is.null(ddd$outlist)) {
       if (ddd$outlist == "minimal") {
          res <- list(b=beta, beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, vb=vb,
                      tau2=tau2,
@@ -808,8 +811,9 @@ correct=TRUE, level=95, digits, verbose=FALSE, ...) {
                      digits=digits,
                      fit.stats=fit.stats)
       } else {
-         res <- eval(parse(text=paste0("list(", ddd$outlist, ")")))
+         res <- eval(str2lang(paste0("list(", ddd$outlist, ")")))
       }
+
    }
 
    time.end <- proc.time()
