@@ -1,7 +1,7 @@
 rma.peto <- function(ai, bi, ci, di, n1i, n2i,
 data, slab, subset,
 add=1/2, to="only0", drop00=TRUE, ### for add/to/drop00, 1st element for escalc(), 2nd for Peto's method
-level=95, digits, verbose=FALSE, ...) {
+level=95, verbose=FALSE, digits, ...) {
 
    #########################################################################
 
@@ -62,13 +62,15 @@ level=95, digits, verbose=FALSE, ...) {
 
    if (verbose > 2) {
       opwarn <- options(warn=1)
-      on.exit(options(warn=opwarn$warn))
+      on.exit(options(warn=opwarn$warn), add=TRUE)
    }
 
    #########################################################################
 
+   if (verbose) .space()
+
    if (verbose)
-      message(mstyle$message("\nExtracting data and computing yi/vi values ..."))
+      message(mstyle$message("Extracting data and computing yi/vi values ..."))
 
    ### check if data argument has been specified
 
@@ -86,25 +88,18 @@ level=95, digits, verbose=FALSE, ...) {
 
    ### extract slab and subset values, possibly from the data frame specified via data (arguments not specified are NULL)
 
-   mf.slab   <- mf[[match("slab",   names(mf))]]
-   mf.subset <- mf[[match("subset", names(mf))]]
-   slab   <- eval(mf.slab,   data, enclos=sys.frame(sys.parent()))
-   subset <- eval(mf.subset, data, enclos=sys.frame(sys.parent()))
+   slab   <- .getx("slab",   mf=mf, data=data)
+   subset <- .getx("subset", mf=mf, data=data)
 
    ### extract/calculate ai,bi,ci,di,n1i,n2i values
 
-   mf.ai  <- mf[[match("ai",  names(mf))]]
-   mf.bi  <- mf[[match("bi",  names(mf))]]
-   mf.ci  <- mf[[match("ci",  names(mf))]]
-   mf.di  <- mf[[match("di",  names(mf))]]
-   mf.n1i <- mf[[match("n1i", names(mf))]]
-   mf.n2i <- mf[[match("n2i", names(mf))]]
-   ai  <- eval(mf.ai,  data, enclos=sys.frame(sys.parent()))
-   bi  <- eval(mf.bi,  data, enclos=sys.frame(sys.parent()))
-   ci  <- eval(mf.ci,  data, enclos=sys.frame(sys.parent()))
-   di  <- eval(mf.di,  data, enclos=sys.frame(sys.parent()))
-   n1i <- eval(mf.n1i, data, enclos=sys.frame(sys.parent()))
-   n2i <- eval(mf.n2i, data, enclos=sys.frame(sys.parent()))
+   ai  <- .getx("ai",  mf=mf, data=data, checknumeric=TRUE)
+   bi  <- .getx("bi",  mf=mf, data=data, checknumeric=TRUE)
+   ci  <- .getx("ci",  mf=mf, data=data, checknumeric=TRUE)
+   di  <- .getx("di",  mf=mf, data=data, checknumeric=TRUE)
+   n1i <- .getx("n1i", mf=mf, data=data, checknumeric=TRUE)
+   n2i <- .getx("n2i", mf=mf, data=data, checknumeric=TRUE)
+
    if (is.null(bi)) bi <- n1i - ai
    if (is.null(di)) di <- n2i - ci
    ni <- ai + bi + ci + di
@@ -112,6 +107,9 @@ level=95, digits, verbose=FALSE, ...) {
    k <- length(ai) ### number of outcomes before subsetting
    k.all <- k
 
+   if (length(ai)==0L || length(bi)==0L || length(ci)==0L || length(di)==0L)
+      stop(mstyle$stop("Must specify arguments ai, bi, ci, di (or ai, ci, n1i, n2i)."))
+
    ids <- seq_len(k)
 
    ### generate study labels if none are specified
@@ -146,16 +144,17 @@ level=95, digits, verbose=FALSE, ...) {
       if (verbose)
          message(mstyle$message("Subsetting ..."))
 
-      subset <- .setnafalse(subset, k=k)
+      subset <- .chksubset(subset, k)
 
-      ai   <- ai[subset]
-      bi   <- bi[subset]
-      ci   <- ci[subset]
-      di   <- di[subset]
-      ni   <- ni[subset]
-      slab <- slab[subset]
-      ids  <- ids[subset]
-      k    <- length(ai)
+      ai   <- .getsubset(ai,   subset)
+      bi   <- .getsubset(bi,   subset)
+      ci   <- .getsubset(ci,   subset)
+      di   <- .getsubset(di,   subset)
+      ni   <- .getsubset(ni,   subset)
+      slab <- .getsubset(slab, subset)
+      ids  <- .getsubset(ids,  subset)
+
+      k <- length(ai)
 
    }
 
@@ -166,9 +165,9 @@ level=95, digits, verbose=FALSE, ...) {
 
    ### calculate observed effect estimates and sampling variances
 
-   dat <- escalc(measure="PETO", ai=ai, bi=bi, ci=ci, di=di, add=add[1], to=to[1], drop00=drop00[1])
-   yi  <- dat$yi ### one or more yi/vi pairs may be NA/NA
-   vi  <- dat$vi ### one or more yi/vi pairs may be NA/NA
+   dat <- .do.call(escalc, measure="PETO", ai=ai, bi=bi, ci=ci, di=di, add=add[1], to=to[1], drop00=drop00[1])
+   yi <- dat$yi ### one or more yi/vi pairs may be NA/NA
+   vi <- dat$vi ### one or more yi/vi pairs may be NA/NA
 
    ### if drop00[2]=TRUE, set counts to NA for studies that have no events (or all events) in both arms
 
@@ -183,10 +182,8 @@ level=95, digits, verbose=FALSE, ...) {
 
    ### save the actual cell frequencies and yi/vi values (including potential NAs)
 
-   ai.f <- ai
-   bi.f <- bi
-   ci.f <- ci
-   di.f <- di
+   outdat.f <- list(ai=ai, bi=bi, ci=ci, di=di)
+
    yi.f <- yi
    vi.f <- vi
    ni.f <- ni
@@ -301,7 +298,7 @@ level=95, digits, verbose=FALSE, ...) {
 
    #########################################################################
 
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   level <- .level(level)
 
    ###### model fitting, test statistics, and confidence intervals
 
@@ -361,7 +358,11 @@ level=95, digits, verbose=FALSE, ...) {
 
    ll.ML     <- -1/2 * (k.yi)   * log(2*base::pi)                   - 1/2 * sum(log(vi))                      - 1/2 * RSS
    ll.REML   <- -1/2 * (k.yi-1) * log(2*base::pi) + 1/2 * log(k.yi) - 1/2 * sum(log(vi)) - 1/2 * log(sum(wi)) - 1/2 * RSS
-   dev.ML    <- -2 * (ll.ML - sum(dnorm(yi, mean=yi, sd=sqrt(vi), log=TRUE)))
+   if (any(vi <= 0)) {
+      dev.ML <- -2 * ll.ML
+   } else {
+      dev.ML <- -2 * (ll.ML - sum(dnorm(yi, mean=yi, sd=sqrt(vi), log=TRUE)))
+   }
    AIC.ML    <- -2 * ll.ML   + 2
    BIC.ML    <- -2 * ll.ML   + log(k.yi)
    AICc.ML   <- -2 * ll.ML   + 2 * max(k.yi, 3) / (max(k.yi, 3) - 2)
@@ -398,7 +399,9 @@ level=95, digits, verbose=FALSE, ...) {
    test      <- "z"
    ddf       <- NA
 
-   if (is.null(ddd$outlist)) {
+   if (is.null(ddd$outlist) || ddd$outlist == "nodata") {
+
+      outdat <- list(ai=ai, bi=bi, ci=ci, di=di)
 
       res <- list(b=beta, beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, vb=vb,
                   tau2=tau2, tau2.f=tau2,
@@ -406,7 +409,7 @@ level=95, digits, verbose=FALSE, ...) {
                   QE=QE, QEp=QEp,
                   k=k, k.f=k.f, k.yi=k.yi, k.pos=k.pos, k.eff=k.eff, k.all=k.all, p=p, p.eff=p.eff, parms=parms,
                   int.only=int.only, intercept=intercept, coef.na=coef.na,
-                  yi=yi, vi=vi, yi.f=yi.f, vi.f=vi.f, X.f=X.f, ai=ai, bi=bi, ci=ci, di=di, ai.f=ai.f, bi.f=bi.f, ci.f=ci.f, di.f=di.f, ni=ni, ni.f=ni.f,
+                  yi=yi, vi=vi, yi.f=yi.f, vi.f=vi.f, X.f=X.f, outdat.f=outdat.f, outdat=outdat, ni=ni, ni.f=ni.f,
                   ids=ids, not.na=not.na, subset=subset, not.na.yivi=not.na.yivi, slab=slab, slab.null=slab.null,
                   measure=measure, method=method, weighted=weighted,
                   test=test, ddf=ddf, dfs=ddf, btt=btt, m=m,
@@ -415,9 +418,11 @@ level=95, digits, verbose=FALSE, ...) {
                   fit.stats=fit.stats,
                   formula.yi=NULL, formula.mods=NULL, version=packageVersion("metafor"), call=mf)
 
-   }
+      if (is.null(ddd$outlist))
+         res <- append(res, list(data=data), which(names(res) == "fit.stats"))
+
+   } else {
 
-   if (!is.null(ddd$outlist)) {
       if (ddd$outlist == "minimal") {
          res <- list(b=beta, beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, vb=vb,
                      tau2=tau2,
@@ -430,8 +435,9 @@ level=95, digits, verbose=FALSE, ...) {
                      digits=digits,
                      fit.stats=fit.stats)
       } else {
-         res <- eval(parse(text=paste0("list(", ddd$outlist, ")")))
+         res <- eval(str2lang(paste0("list(", ddd$outlist, ")")))
       }
+
    }
 
    time.end <- proc.time()
