@@ -1,8 +1,8 @@
-simulate.rma <- function(object, nsim = 1, seed = NULL, olim, ...) {
+simulate.rma <- function(object, nsim=1, seed=NULL, olim, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   .chkclass(class(object), must="rma", notav=c("rma.glmm", "rma.mh", "rma.peto", "rma.uni.selmodel"))
+   .chkclass(class(object), must="rma", notav=c("rma.gen", "rma.glmm", "rma.mh", "rma.peto", "rma.uni.selmodel"))
 
    na.act <- getOption("na.action")
 
@@ -18,9 +18,14 @@ simulate.rma <- function(object, nsim = 1, seed = NULL, olim, ...) {
       R.seed <- get(".Random.seed", envir = .GlobalEnv)
       set.seed(seed)
       RNGstate <- structure(seed, kind = as.list(RNGkind()))
-      on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
+      on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv), add=TRUE)
    }
 
+   nsim <- round(nsim)
+
+   if (nsim <= 0)
+      stop(mstyle$stop("Argument 'nsim' must be >= 1."))
+
    #########################################################################
 
    ### fitted values
