@@ -2,8 +2,7 @@ summary.rma <- function(object, digits, showfit=TRUE, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(object, "rma"))
-      stop(mstyle$stop("Argument 'object' must be an object of class \"rma\"."))
+   .chkclass(class(object), must="rma")
 
    if (missing(digits)) {
       digits <- .get.digits(xdigits=object$digits, dmiss=TRUE)
