@@ -1,6 +1,8 @@
 ### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true")
 
-### see also: http://www.metafor-project.org/doku.php/plots:funnel_plot_with_trim_and_fill
+### see also: https://www.metafor-project.org/doku.php/plots:funnel_plot_with_trim_and_fill
+
+source("settings.r")
 
 context("Checking plots example: funnel plot with trim and fill")
 
@@ -15,9 +17,6 @@ test_that("plot can be drawn.", {
    ### decrease margins so the full space is used
    par(mar=c(5,4,1,2))
 
-   ### load ETS data
-   data(dat.hackshaw1998, package="metafor")
-
    ### fit random-effects model
    res <- rma(yi, vi, data=dat.hackshaw1998, measure="OR")
 
@@ -32,3 +31,5 @@ test_that("plot can be drawn.", {
    out <- capture.output(print(taf))
 
 })
+
+rm(list=ls())
