@@ -2,6 +2,8 @@
 
 ### see also: https://www.metafor-project.org/doku.php/plots:plot_of_cumulative_results
 
+source("settings.r")
+
 context("Checking plots example: plot of cumulative results")
 
 test_that("plot can be drawn.", {
@@ -15,9 +17,6 @@ test_that("plot can be drawn.", {
    ### decrease margins so the more space is used
    par(mar=c(5,5,2,2))
 
-   ### load BCG vaccine data
-   data(dat.bcg, package="metafor")
-
    ### calculate log risk ratios and corresponding sampling variances
    dat <- escalc(measure="RR", ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
 
@@ -25,7 +24,7 @@ test_that("plot can be drawn.", {
    res <- rma(yi, vi, data=dat)
 
    ### cumulative meta-analysis (in the order of publication year)
-   tmp <- cumul(res, order=dat$year)
+   tmp <- cumul(res, order=year)
 
    ### plot of cumulative results
    plot(tmp, transf=exp, xlim=c(.25,.5), lwd=3, cex=1.3)
@@ -33,3 +32,5 @@ test_that("plot can be drawn.", {
    par(opar)
 
 })
+
+rm(list=ls())
