@@ -2,6 +2,8 @@
 
 ### see also: https://www.metafor-project.org/doku.php/plots:radial_plot
 
+source("settings.r")
+
 context("Checking plots example: radial (Galbraith) plot")
 
 test_that("plot can be drawn.", {
@@ -15,11 +17,8 @@ test_that("plot can be drawn.", {
    ### adjust margins so the space is better used
    par(mar=c(5,4,0,2))
 
-   ### load ETS data
-   data(dat.hackshaw1998, package="metafor")
-
-   ### fit fixed-effects model
-   res <- rma(yi, vi, data=dat.hackshaw1998, method="FE")
+   ### fit equal-effects model
+   res <- rma(yi, vi, data=dat.hackshaw1998, method="EE")
 
    ### draw radial plot
    radial(res)
@@ -27,3 +26,5 @@ test_that("plot can be drawn.", {
    par(opar)
 
 })
+
+rm(list=ls())
